//
//  DrawingSurface.m
//  TheOvalApp
//
//  Created by Benny Michielsen on 05/06/08.
//  Copyright 2008 __MyCompanyName__. All rights reserved.
//

#import "DrawingSurface.h"
#import "Oval.h"

@implementation DrawingSurface

- (id)initWithFrame:(NSRect)frame {
    self = [super initWithFrame:frame];
    if (self) {
        startPoint = NSZeroPoint;
		endPoint = NSZeroPoint;
    }
    return self;
}

- (void)drawRect:(NSRect)rect {
    // Drawing code here.
	for(Oval *o in  ovals){
		NSBezierPath *path = [NSBezierPath bezierPathWithOvalInRect:[o ovalRect]];
		[[NSColor yellowColor] set];
		[path fill];
	}
}

- (NSRect)currentRect
{
	float minX = MIN(startPoint.x, endPoint.x);
	float maxX = MAX(startPoint.x, endPoint.x);
	float minY = MIN(startPoint.y, endPoint.y);
	float maxY = MAX(startPoint.y, endPoint.y);
	return NSMakeRect(minX, minY, maxX - minX, maxY-minY);
}

- (BOOL)equalToZeroPoint:(NSPoint )p
{
	return (p.x == NSZeroPoint.x && p.y == NSZeroPoint.y);
}

#pragma mark Events
- (void)mouseDown:(NSEvent *)event
{
	NSPoint point = [self convertPoint:[event locationInWindow] fromView:nil];
	NSLog(@"Recording startpoint = %@",NSStringFromPoint(point));
	startPoint = point;
	[self setNeedsDisplay:YES];
}

- (void)mouseDragged:(NSEvent *)event
{
	NSPoint point = [self convertPoint:[event locationInWindow] fromView:nil];
	NSLog(@"Dragging with mouse, current endpoint = %@",NSStringFromPoint(point));
	endPoint = point;
	[self autoscroll:event];
	[self setNeedsDisplay:YES];
}

- (void)mouseUp:(NSEvent *)event
{
	NSPoint point = [self convertPoint:[event locationInWindow] fromView:nil];
	NSLog(@"Recording stopped, endpoint = %@",NSStringFromPoint(point));
	endPoint = point;
	if((![self equalToZeroPoint:startPoint]) || (![self equalToZeroPoint:endPoint])){
		Oval *newOval = [[Oval alloc] init:[self currentRect]];
		startPoint = NSZeroPoint;
		endPoint = NSZeroPoint;
		
		NSUndoManager *undo = [self undoManager];
		// Has an edit occured already in this event?
		if([undo groupingLevel])
		{
			// Close the last group
			[undo endUndoGrouping];
			// Open a new group
			[undo beginUndoGrouping];
		}
		
		[ovalsController addObject:newOval];
		
	}
}

#pragma mark Methods

- (void)insertObject:(Oval *)o inOvalsAtIndex:(int)index
{
	NSLog(@"adding %@ to %@", o, [self ovals]);
	NSUndoManager *undo = [self undoManager];
	[[undo prepareWithInvocationTarget:self] removeObjectFromOvalsAtIndex:index];
	if(![undo isUndoing])
	{
		[undo setActionName:@"Insert Oval"];
	}
	[ovals insertObject:o atIndex:index];	 
	[self setNeedsDisplay:YES];
}

- (void)removeObjectFromOvalsAtIndex:(int)index
{
	Oval *o =  [ovals objectAtIndex:index];
	NSLog(@"removing %@ from %@", o, ovals);
	NSUndoManager *undo = [self undoManager];
	[[undo prepareWithInvocationTarget:self] insertObject:o inOvalsAtIndex:index];
	//if(![undo isUndoing])
	//{
	//	[undo setActionName:@"Delete Oval"];
	//}
	[ovals removeObjectAtIndex:index];
	[self setNeedsDisplay:YES];
}

@synthesize ovals;

@end
